VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClsDBProps"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'---------------------------------------------------------------------------------
'
'View database tables, system tables, fields, indexes, queries and properties for
'a given Microsoft Access database.
'
'
'*************************************************************************************
' As usual when updating databases at low level, changes may be very difficult
' to undo. Unless you know exactly what you are doing, create a backup of the database
' prior to modifying any properties.
'*************************************************************************************
'
'You may recognise some of the code from the VisData application which is
'included in Visual Basic.
'
'Disclaimer:  The DPProps class is provided "as-is" by the Author,
'who is not responsible for any harm to computer systems or software
'with which it is used. You are free to modify the code and/or use it
'yourself in any way you see fit.
'You are free to redistribute the package under the following conditions:
'1) appropriate credit is given to author,
'2) the code is provided unaltered and in full, and
'3) you do not charge any kind of fee for the code or its binary version
'   without the express permission of the author.
'
' If you change this code please, please, p-l-e-a-s-e send me
' a copy. I have just strated using classes and any help will be appriciated.
'
' Please email your comments to paulo.batista@lineone.net
'
'---------------------------------------------------------------------------------
'
'
' Node identification
Const ROOT_STR = "RootNode"
Const TABLE_STR = "Table"
Const ATTACHED_STR = "Attached"
Const QUERY_STR = "Query"
Const FIELD_STR = "Field"
Const FIELDS_STR = "Fields"
Const INDEX_STR = "Index"
Const INDEXES_STR = "Indexes"
Const PROPERTY_STR = "Property"
Const PROPERTIES_STR = "Properties"

Private gnodDBNode    As Node        'current database node in treeview

' -----------------------------------------------------------------------------------------
' |
' |  PUBLIC SUBS
' |
' -----------------------------------------------------------------------------------------
Public Sub Initialise(DataSource As Database, tvName As TreeView, Optional IncludeSysTables As Boolean = False)
'
' Creates first and second level nodes (assumes that the database has been initialised)
'
'   Accepts
'   =======
'   DataSource       - the variable that references the access database on your form
'   tvName           - the TreeView's name
'   IncludeSysTables - if set to true it will display the MS-Access system tables
'
  Dim nodX As Node
  Dim sTBLName As String
  Dim sQRYName As String
  Dim sPropName As String
  Dim tblObj As DAO.TableDef
  Dim qdfObj As DAO.QueryDef
  Dim prpObj As DAO.Property
  Dim bAttached As Boolean
  Dim sTmp As String
  Dim qryObj As QueryDef
  Dim bTablesFound As Boolean
  Dim bIncludeSysTables As Boolean

  On Error GoTo Err_LoadDatabase

  Screen.MousePointer = vbHourglass
  tvName.Nodes.Clear
  
  'add root node
  Set nodX = tvName.Nodes.Add(, , "R" & DataSource.Name, DataSource.Name, ROOT_STR)
  nodX.Tag = ROOT_STR
  
  'add the table properties
  Set nodX = tvName.Nodes.Add("R" & DataSource.Name, tvwChild, _
                                      PROPERTIES_STR, PROPERTIES_STR, PROPERTY_STR)
  nodX.Tag = PROPERTIES_STR
  NodeClick DataSource, tvName, nodX
  nodX.Expanded = False
  
  ' System tables
  bIncludeSysTables = IncludeSysTables
  
  'add the tables
  For Each tblObj In DataSource.TableDefs
    If (tblObj.Attributes And dbSystemObject) = 0 Or bIncludeSysTables Then
      sTBLName = tblObj.Name
      bTablesFound = True
      If (tblObj.Attributes And dbAttachedTable) = dbAttachedTable Then
        bAttached = True
      ElseIf (tblObj.Attributes And dbAttachedODBC) = dbAttachedODBC Then
        bAttached = True
      Else
        bAttached = False
      End If
      
      If bAttached Then
        Set nodX = tvName.Nodes.Add("R" & DataSource.Name, tvwChild, _
                                      tblObj.Name, tblObj.Name, ATTACHED_STR)
        
      Else
        Set nodX = tvName.Nodes.Add("R" & DataSource.Name, tvwChild, _
                                      tblObj.Name, tblObj.Name, TABLE_STR)
      End If
      nodX.Tag = TABLE_STR
      Set nodX = tvName.Nodes.Add(sTBLName, tvwChild, _
                                      sTBLName & ">Fields", _
                                      FIELDS_STR, FIELD_STR)
      nodX.Tag = FIELDS_STR
      Set nodX = tvName.Nodes.Add(sTBLName, tvwChild, _
                                      sTBLName & ">Indexes", _
                                      INDEXES_STR, INDEX_STR)
      nodX.Tag = INDEXES_STR
      Set nodX = tvName.Nodes.Add(sTBLName, tvwChild, _
                                      sTBLName & ">" & PROPERTIES_STR, _
                                      PROPERTIES_STR, PROPERTY_STR)
      nodX.Tag = PROPERTIES_STR
      If bAttached Then
        'add node to show attachment details
        sTmp = DataSource.TableDefs(sTBLName).Connect
        sTmp = Left(sTmp, InStr(sTmp, ";") - 1)
        If Len(sTmp) = 0 Then
          sTmp = gsMSACCESS
        End If
        Set nodX = tvName.Nodes.Add(sTBLName, tvwChild, _
                                        sTBLName & ">AttachType", _
                                        sTmp & " Table", ATTACHED_STR)
      End If
    End If
  Next


  'add the querydefs
  For Each qryObj In DataSource.QueryDefs
    sQRYName = qryObj.Name
    Set nodX = tvName.Nodes.Add("R" & DataSource.Name, tvwChild, sQRYName, sQRYName, QUERY_STR)
    nodX.Tag = QUERY_STR
    
    Set nodX = tvName.Nodes.Add(sQRYName, tvwChild, _
                                   sQRYName & ">" & PROPERTIES_STR, _
                                   PROPERTIES_STR, PROPERTY_STR)
    nodX.Tag = PROPERTIES_STR
  Next

  Screen.MousePointer = vbDefault
  Exit Sub
  
Err_LoadDatabase:
  ShowError
End Sub

Public Sub NodeClick(DataSource As Database, tvName As TreeView, ByVal Node As MSComctlLib.Node)
'
' Creates child nodes (assumes that the database has been initialised)
'
'   Accepts
'   =======
'   DataSource  - the variable that references the access database on your form
'   tvName      - the TreeView's name
'   Node        - pointer to the node clicked
'

  On Error GoTo tvName_NodeClickErr
  
  Dim nod As Node
  Dim nodX As Node
  Dim fldObj As DAO.Field
  Dim idxObj As DAO.Index
  Dim prpObj As DAO.Property
  Dim colTmp As Object
  Dim vTmp As Variant

  Set gnodDBNode = Node

  Select Case Node.Tag
    Case FIELDS_STR
      If Node.Children > 0 Then Exit Sub
      'add the fields
      For Each fldObj In DataSource.TableDefs(Node.Parent.Text).Fields
        Set nodX = tvName.Nodes.Add(Node.Key, _
                                       tvwChild, _
                                       Node.Parent.Key & ">" & FIELDS_STR & ">" & fldObj.Name, _
                                       fldObj.Name, FIELD_STR)
        nodX.Tag = FIELD_STR
      Next
      Node.Expanded = True
      
    Case FIELD_STR
      If Node.Children > 0 Then Exit Sub
      For Each prpObj In DataSource.TableDefs(Node.Parent.Parent.Text).Fields(Node.Text).Properties
        'special case - the Value property is not available from the field object on a tabledef
        If prpObj.Name <> "Value" Then
          vTmp = PropertyValue(prpObj)
          Set nodX = tvName.Nodes.Add(Node.Key, _
                                         tvwChild, _
                                         Node.Parent.Key & Node.Key & ">" & prpObj.Name, _
                                         prpObj.Name & "=" & vTmp, PROPERTY_STR)
          nodX.Tag = PROPERTY_STR
        End If
      Next
      Node.Expanded = True
      Set tvName.SelectedItem = Node
        
    Case INDEXES_STR
      If Node.Children > 0 Then Exit Sub
      'add the indexes
      For Each idxObj In DataSource.TableDefs(Node.Parent.Text).Indexes
        Set nodX = tvName.Nodes.Add(Node.Key, _
                                       tvwChild, _
                                       Node.Parent.Key & ">" & INDEXES_STR & ">" & idxObj.Name, _
                                       idxObj.Name, INDEX_STR)
        nodX.Tag = INDEX_STR
      Next
      Node.Expanded = True
      
    Case INDEX_STR
      If Node.Children > 0 Then Exit Sub
      For Each prpObj In DataSource.TableDefs(Node.Parent.Parent.Text).Indexes(Node.Text).Properties
        vTmp = PropertyValue(prpObj)
        Set nodX = tvName.Nodes.Add(Node.Key, _
                                       tvwChild, _
                                       Node.Parent.Key & Node.Key & ">" & prpObj.Name, _
                                       prpObj.Name & "=" & vTmp, PROPERTY_STR)
        nodX.Tag = PROPERTY_STR
      Next
      Node.Expanded = True
      Set tvName.SelectedItem = Node
    
    Case PROPERTIES_STR
      If Node.Children > 0 Then Exit Sub
      'add the properties
      If Node.Parent.Parent Is Nothing Then
        Set colTmp = DataSource.Properties
      Else
        Select Case Node.Parent.Tag
          Case TABLE_STR
            Set colTmp = DataSource.TableDefs(Node.Parent.Text).Properties
          Case QUERY_STR
            Set colTmp = DataSource.QueryDefs(Node.Parent.Text).Properties
          Case PROPERTY_STR
            Exit Sub  'undone: need to get parent object
        End Select
      End If
      For Each prpObj In colTmp
        vTmp = PropertyValue(prpObj)
        If VarType(vTmp) = vbString Then
          'truncate it to 50 chars
          vTmp = Left$(vTmp, 50)
        End If
        If Node.Parent Is Nothing Then
          Set nodX = tvName.Nodes.Add(Node.Key, _
                                         tvwChild, _
                                         Node.Key & ">" & prpObj.Name, _
                                         prpObj.Name & "=" & vTmp, PROPERTY_STR)
        Else
          Set nodX = tvName.Nodes.Add(Node.Key, _
                                         tvwChild, _
                                         Node.Parent.Key & ">" & prpObj.Name, _
                                         prpObj.Name & "=" & vTmp, PROPERTY_STR)
        End If
        nodX.Tag = PROPERTY_STR
      Next
      Node.Expanded = True
    
  End Select
  
  Exit Sub
tvName_NodeClickErr:
  If Err = 35602 Then Resume Next
  ShowError
End Sub

Public Sub NodeDoubleClick(DataSource As Database)
'
'  (assumes that the database has been initialised)
'
'   Accepts
'   =======
'   DataSource  - the variable that references the access database on your form

  If gnodDBNode Is Nothing Then Exit Sub
  
  If gnodDBNode.Tag = PROPERTY_STR Then
     SetPropertyValues DataSource
  End If
  
End Sub

' -----------------------------------------------------------------------------------------
' |
' |  PRIVATE SUBS
' |
' -----------------------------------------------------------------------------------------

Private Sub SetPropertyValues(DataSource As Database)
  
  On Error GoTo mnuDBPUEdit_ClickErr
  
  Dim prpObj As Property
  Dim vTmp As Variant
  Dim vNew As Variant
  Dim Result As Variant
  
  If gnodDBNode.Parent.Parent.Parent Is Nothing Then
    'must be a database property
    Set prpObj = DataSource.Properties(VBA.Left(gnodDBNode.Text, InStr(gnodDBNode.Text, "=") - 1))
  Else
    Select Case gnodDBNode.Parent.Parent.Tag
      Case TABLE_STR
        Set prpObj = DataSource.TableDefs(gnodDBNode.Parent.Parent.Text).Properties(VBA.Left(gnodDBNode.Text, InStr(gnodDBNode.Text, "=") - 1))
      Case QUERY_STR
        Set prpObj = DataSource.QueryDefs(gnodDBNode.Parent.Parent.Text).Properties(VBA.Left(gnodDBNode.Text, InStr(gnodDBNode.Text, "=") - 1))
      Case FIELDS_STR
        Set prpObj = DataSource.TableDefs(gnodDBNode.Parent.Parent.Parent.Text).Fields(gnodDBNode.Parent.Text).Properties(VBA.Left(gnodDBNode.Text, InStr(gnodDBNode.Text, "=") - 1))
      Case INDEXES_STR
        Set prpObj = DataSource.TableDefs(gnodDBNode.Parent.Parent.Parent.Text).Indexes(gnodDBNode.Parent.Text).Properties(VBA.Left(gnodDBNode.Text, InStr(gnodDBNode.Text, "=") - 1))
      Case Else
        Exit Sub
    End Select
  End If

  'store the value
  vTmp = prpObj.Value
  
  On Error Resume Next
  'try to set it to it's current value to see if it is readonly
  prpObj.Value = vTmp
  If Err Then
    'readonly so just exit
    Err.Clear
    MsgBox "'" & prpObj.Name & "'" & " Property is Read Only!", vbExclamation
    Exit Sub
  End If
  On Error GoTo mnuDBPUEdit_ClickErr
  
Re_Enter:
  Result = InputBox("Enter property value:", prpObj.Name & " Property Value", prpObj.Value)
  If Len(Result) > 0 Then
    If prpObj.Type = dbBoolean Then
      On Error Resume Next
      'try to set it to it'svalue to see if it is valid
      prpObj.Value = CBool(Result)
      If Err Then
        'readonly so just exit
        Err.Clear
        MsgBox "'" & prpObj.Name & "'" & " Property can not accept " & Result & "!", vbExclamation
        GoTo Re_Enter
      End If
      gnodDBNode.Text = prpObj.Name & "=" & prpObj.Value
      On Error GoTo mnuDBPUEdit_ClickErr
    Else
      prpObj.Value = Result
      gnodDBNode.Text = prpObj.Name & "=" & prpObj.Value
    End If
  End If
  'see if it was a Name property
  If prpObj.Name = "Name" Then
    gnodDBNode.Parent.Text = prpObj.Value
  End If
  
  Exit Sub
mnuDBPUEdit_ClickErr:
  ShowError
End Sub

Private Sub ShowError()
'------------------------------------------------------------
'
' This sub displays the error message with it's Err code
'   if DBEngine error adds the prompt to display the error
'   collection
'
'------------------------------------------------------------

  Screen.MousePointer = vbDefault
  'MsgBox vbNullString, False

  sTmp = "The following Error occurred:" & vbCrLf & vbCrLf
  'add the error string
  sTmp = sTmp & Err.Description & vbCrLf
  'add the error number
  sTmp = sTmp & "Number: " & Err
  Beep
  MsgBox sTmp

End Sub

' -----------------------------------------------------------------------------------------
' |
' |  PRIVATE PROPERTY
' |
' -----------------------------------------------------------------------------------------

Private Property Get PropertyValue(prpObj As DAO.Property) As Variant
'
'   Accepts
'   =======
'   prpObj  - the database property
'
'   Returns
'   =======
'   PropertyValue - current propertie's [prpObj]  value ( boolean / string / integer / etc )
  
  On Error Resume Next
  
  Dim vTmp As Variant
  
  vTmp = prpObj.Value
  If Err Then
    Err.Clear
    PropertyValue = "N/A"
  Else
    PropertyValue = vTmp
  End If
  
End Property

